@@ -13,6 +13,8 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
+import math
+import os
 import urllib.parse
 from time import sleep
 from lxml import etree
@@ -87,7 +89,7 @@ class Events:
         :return: 成功返回True，否则为False
         """
         return self.__reply_status_check(self.device.con.get(path="setDisplayState", args="state=0")) and \
-               self.__reply_status_check(self.device.con.get(path="setDisplayState", args="state=2"))
+            self.__reply_status_check(self.device.con.get(path="setDisplayState", args="state=2"))
 
     def get_display_state(self) -> DisplayState:
         """
@@ -142,8 +144,8 @@ class Events:
         :param delay: 点击延时时间(单位:毫秒)，默认无延时
         :return: 成功返回True，否则为False
         """
-        return self.__reply_status_check(self.device.con.get(path="sendTouchEvent", args="points=" + str(point.x)
-                                                                                         + "|" + str(point.y)
+        return self.__reply_status_check(self.device.con.get(path="sendTouchEvent", args="points=" + str(int(point.x))
+                                                                                         + "|" + str(int(point.y))
                                                                                          + "&delay=" + str(delay)))
 
     def multi_click(self, points: List[Point], delay: int = 0) -> bool:
@@ -155,7 +157,7 @@ class Events:
         """
         args = ""
         for point in points:
-            args += str(point.x) + "|" + str(point.y)
+            args += str(int(point.x)) + "|" + str(int(point.y))
             if points.index(point) != len(points) - 1:
                 args += ","
         return self.__reply_status_check(self.device.con.get(path="sendTouchEvent", args="points=" + args
@@ -168,10 +170,112 @@ class Events:
         :param p2: 坐标点类Point对象，终点
         :return: 成功返回True，否则为False
         """
-        return self.__reply_status_check(self.device.con.get(path="sendSlideEvent", args="sliders=" + str(p1.x)
-                                                                                         + "|" + str(p1.y)
-                                                                                         + "->" + str(p2.x)
-                                                                                         + "|" + str(p2.y)))
+        return self.__reply_status_check(self.device.con.get(path="sendSlideEvent", args="sliders=" + str(int(p1.x))
+                                                                                         + "|" + str(int(p1.y))
+                                                                                         + "->" + str(int(p2.x))
+                                                                                         + "|" + str(int(p2.y))))
+
+    def touch_down(self, point: Point, slot_id: int = 9) -> bool:
+        """
+        touch按下。\n
+        :param point: 坐标点类Point对象
+        :param slot_id: 用于区分多指触摸的编号
+        :return: 成功返回True，否则为False
+        """
+        return self.__reply_status_check(self.device.con.get(path="sendTouchEventPWithUInput",
+                                                             args="type=1&slot=" + str(slot_id) + "&id=" + str(slot_id) + "&x=" + str(int(point.x))
+                                                                  + "&y=" + str(int(point.y))))
+
+    def touch_up(self, slot_id: int = 9) -> bool:
+        """
+        touch抬起。\n
+        :param slot_id: 用于区分多指触摸的编号
+        :return: 成功返回True，否则为False
+        """
+        return self.__reply_status_check(self.device.con.get(path="sendTouchEventPWithUInput",
+                                                             args="type=0&slot=" + str(slot_id) + "&id=0&x=0&y=0"))
+
+    def drag(self, p1: Point, p2: Point, delay: int = 1) -> bool:
+        """
+        拖动。\n
+        :param p1: 坐标点类Point对象，起始点
+        :param p2: 坐标点类Point对象，终点
+        :param delay: 起始点长按时间，默认1秒
+        :return: 成功返回True，否则为False
+        """
+        begin = self.__reply_status_check(
+            self.device.con.get(path="sendTouchEventPWithUInput",
+                                args="type=1&slot=9&id=9&x=" + str(int(p1.x)) + "&y=" + str(int(p1.y))))
+        sleep(delay)
+        distance = math.sqrt((p1.x - p2.x) ** 2 + (p1.y - p2.y) ** 2)
+        if distance > 30:
+            steps = 30
+        elif distance < 2:
+            steps = 2
+        else:
+            steps = distance
+        for j in range(steps):
+            if p1.x < p2.x:
+                tx = p1.x + ((p2.x - p1.x) / steps * j)
+            else:
+                tx = p1.x - ((p1.x - p2.x) / steps * j)
+            if p1.y < p2.y:
+                ty = p1.y + ((p2.y - p1.y) / steps * j)
+            else:
+                ty = p1.y - ((p1.y - p2.y) / steps * j)
+            self.device.con.get(path="sendTouchEventPWithUInput",
+                                args="type=2&slot=9&id=9&x=" + str(int(tx)) + "&y=" + str(int(ty)))
+        end = self.__reply_status_check(
+            self.device.con.get(path="sendTouchEventPWithUInput",
+                                args="type=0&slot=9&id=9&x=" + str(int(p2.x)) + "&y=" + str(int(p2.y))))
+        return begin and end
+
+    def route_drag(self, pl: List[Point], delay: int = 1) -> bool:
+        """
+        沿路径滑动屏幕。\n
+        :param pl: 坐标点类Point对象列表，起始点->途经点1->...途经点n->终止点，至少两个点
+        :param delay: 起始点长按时间，默认1秒
+        :return: 成功返回True，否则为False
+        """
+        if len(pl) < 2:
+            return False
+        begin = self.__reply_status_check(
+            self.device.con.get(path="sendTouchEventPWithUInput",
+                                args="type=1&slot=9&id=9&x=" + str(int(pl[0].x)) + "&y=" + str(int(pl[0].y))))
+        sleep(delay)
+        for i in range(len(pl) - 1):
+            p1 = pl[i]
+            p2 = pl[i + 1]
+            distance = math.sqrt((p1.x - p2.x) ** 2 + (p1.y - p2.y) ** 2)
+            if distance > 30:
+                steps = 30
+            elif distance < 2:
+                steps = 2
+            else:
+                steps = distance
+            for j in range(steps):
+                if p1.x < p2.x:
+                    tx = p1.x + ((p2.x - p1.x) / steps * j)
+                else:
+                    tx = p1.x - ((p1.x - p2.x) / steps * j)
+                if p1.y < p2.y:
+                    ty = p1.y + ((p2.y - p1.y) / steps * j)
+                else:
+                    ty = p1.y - ((p1.y - p2.y) / steps * j)
+                self.device.con.get(path="sendTouchEventPWithUInput",
+                                    args="type=2&slot=9&id=9&x=" + str(int(tx)) + "&y=" + str(int(ty)))
+        end = self.__reply_status_check(
+            self.device.con.get(path="sendTouchEventPWithUInput",
+                                args="type=0&slot=9&id=9&x=" + str(int(pl[len(pl) - 1].x)) + "&y=" + str(int(pl[len(pl) - 1].y))))
+        return begin and end
+
+    def route_swipe(self, pl: List[Point]) -> bool:
+        """
+        沿路径滑动屏幕。\n
+        :param pl: 坐标点类Point对象列表，起始点->途经点1->...途经点n->终止点，至少两个点
+        :return: 成功返回True，否则为False
+        """
+        return self.route_drag(pl, 0)
 
     def multi_swipe(self, points1: List[Point], points2: List[Point]) -> bool:
         """
@@ -182,9 +286,9 @@ class Events:
         """
         args = ""
         for point1 in points1:
-            args += str(point1.x) + "|" + str(point1.y) \
-                    + "->" + str(points2[points1.index(point1)].x) \
-                    + "|" + str(points2[points1.index(point1)].y)
+            args += str(int(point1.x)) + "|" + str(int(point1.y)) \
+                    + "->" + str(int(points2[points1.index(point1)].x)) \
+                    + "|" + str(int(points2[points1.index(point1)].y))
             if points1.index(point1) != len(points1) - 1:
                 args += ","
         return self.__reply_status_check(self.device.con.get(path="sendSlideEvent", args="sliders=" + args))
@@ -197,6 +301,122 @@ class Events:
         """
         return self.__reply_status_check(self.device.con.get(path="sendPowerKeyEvent", args="delay=" + str(delay)))
 
+    def backspace(self, delay: int = 0) -> bool:
+        """
+        点击退格按键。\n
+        :param delay: 点击延时时间(单位:毫秒)，默认无延时
+        :return: 成功返回True，否则为False
+        """
+        return self.__reply_status_check(self.device.con.get(path="sendKeyEventWithUInput",
+                                                             args="value=14&delay=" + str(delay)))
+
+    def __key_with_control(self, value:  int) -> bool:
+        s1 = self.__reply_status_check(self.device.con.get(path="sendKeyEventPWithUInput", args="value=29&type=1"))
+        sleep(0.1)
+        s2 = self.__reply_status_check(self.device.con.get(path="sendKeyEventWithUInput", args="value=" + str(value)))
+        sleep(0.1)
+        s3 = self.__reply_status_check(self.device.con.get(path="sendKeyEventPWithUInput", args="value=29&type=0"))
+        return s1 and s2 and s3
+
+    def __key_with_shift(self, value:  int) -> bool:
+        s1 = self.__reply_status_check(self.device.con.get(path="sendKeyEventPWithUInput", args="value=42&type=1"))
+        sleep(0.1)
+        s2 = self.__reply_status_check(self.device.con.get(path="sendKeyEventWithUInput", args="value=" + str(value)))
+        sleep(0.1)
+        s3 = self.__reply_status_check(self.device.con.get(path="sendKeyEventPWithUInput", args="value=42&type=0"))
+        return s1 and s2 and s3
+
+    def select_all(self) -> bool:
+        """
+        全选，模拟键盘的Ctrl+A事件\n
+        :return: 成功返回True，否则为False
+        """
+        return self.__key_with_control(30)
+
+    def cut(self) -> bool:
+        """
+        剪切，模拟键盘的Ctrl+X事件\n
+        :return: 成功返回True，否则为False
+        """
+        return self.__key_with_control(45)
+
+    def copy(self) -> bool:
+        """
+        复制，模拟键盘的Ctrl+C事件\n
+        :return: 成功返回True，否则为False
+        """
+        return self.__key_with_control(46)
+
+    def paste(self) -> bool:
+        """
+        粘贴，模拟键盘的Ctrl+V事件\n
+        :return: 成功返回True，否则为False
+        """
+        return self.__key_with_control(47)
+
+    def up(self) -> bool:
+        """
+        方向键-上\n
+        :return: 成功返回True，否则为False
+        """
+        return self.__reply_status_check(self.device.con.get(path="sendKeyEventWithUInput", args="value=103"))
+
+    def enter(self) -> bool:
+        """
+        回车\n
+        :return: 成功返回True，否则为False
+        """
+        return self.__reply_status_check(self.device.con.get(path="sendKeyEventWithUInput", args="value=28"))
+
+    def down(self) -> bool:
+        """
+        方向键-下\n
+        :return: 成功返回True，否则为False
+        """
+        return self.__reply_status_check(self.device.con.get(path="sendKeyEventWithUInput", args="value=108"))
+
+    def left(self) -> bool:
+        """
+        方向键-左\n
+        :return: 成功返回True，否则为False
+        """
+        return self.__reply_status_check(self.device.con.get(path="sendKeyEventWithUInput", args="value=105"))
+
+    def right(self) -> bool:
+        """
+        方向键-右\n
+        :return: 成功返回True，否则为False
+        """
+        return self.__reply_status_check(self.device.con.get(path="sendKeyEventWithUInput", args="value=106"))
+
+    def shift_up(self) -> bool:
+        """
+        shift加方向键-上\n
+        :return: 成功返回True，否则为False
+        """
+        return self.__key_with_shift(103)
+
+    def shift_down(self) -> bool:
+        """
+        shift加方向键-下\n
+        :return: 成功返回True，否则为False
+        """
+        return self.__key_with_shift(108)
+
+    def shift_left(self) -> bool:
+        """
+        shift加方向键-左\n
+        :return: 成功返回True，否则为False
+        """
+        return self.__key_with_shift(105)
+
+    def shift_right(self) -> bool:
+        """
+        shift加方向键-右\n
+        :return: 成功返回True，否则为False
+        """
+        return self.__key_with_shift(106)
+
     def back(self, delay: int = 0) -> bool:
         """
         点击设备返回按键。\n
@@ -205,19 +425,26 @@ class Events:
         """
         return self.__reply_status_check(self.device.con.get(path="sendBackKeyEvent", args="delay=" + str(delay)))
 
-    def home(self, delay: int = 0, timeout: int = None) -> bool:
+    def home(self, delay: int = 0) -> bool:
         """
         点击设备主屏幕按键。\n
         :param delay: 点击延时时间(单位:毫秒)，默认无延时
+        :return: 成功返回True，否则为False
+        """
+        return self.__reply_status_check(self.device.con.get(path="sendHomeKeyEvent", args="delay=" + str(delay)))
+
+    def go_home(self, timeout: int = None) -> bool:
+        """
+        去到桌面。\n
         :param timeout: 超时时间(单位:秒)，默认为框架超时时间
         :return: 成功返回True，否则为False
         """
-        if delay > 0:
-            return self.__reply_status_check(self.device.con.get(path="sendHomeKeyEvent", args="delay=" + str(delay)))
         if not timeout:
             timeout = self.device.default_timeout
         die_time = int(time.time()) + timeout
         while int(time.time()) < die_time:
+            self.device.set_display_on()
+            self.device.unlock()
             if self.__reply_status_check(self.device.con.get(path="sendHomeKeyEvent")):
                 for i in range(0, 10):
                     try:
@@ -304,14 +531,33 @@ class Events:
             raise Exception('error: the file path format is incorrect, and the transfer folder is not supported')
         if remote_path.split("/")[len(remote_path.split("/")) - 1] == "":
             remote_path += file_name
+        _fi = self.device.get_framework_info()
+        if _fi != {} and _fi['version_build'] >= 241010:
+            header = {
+                "FilePath": remote_path,
+                "FileName": file_name,
+                "TotalSize": os.path.getsize(file_path)
+            }
+            re = True
+            with open(file_path, 'rb') as f:
+                while True:
+                    header["Start"] = str(f.tell())
+                    chunk = f.read(1024 * 1024)
+                    if not chunk:
+                        break
+                    else:
+                        re = re and bool(int(str(
+                            self.device.con.post(path="upLoadFile", headers=header, data=chunk, timeout=timeout).read(),
+                            'utf-8')))
+            return re
         header = {
-            "content-type": "application/json",
             "FileName": remote_path
         }
         f = open(file_path, 'rb')
         data = {'file': (file_name, f.read())}
         f.close()
         encode_data = encode_multipart_formdata(data)
+
         data = encode_data[0]
         header['Content-Type'] = encode_data[1]
         return bool(int(str(
@@ -424,13 +670,18 @@ class Events:
         """
         return bool(int(str(self.device.con.get(path="mkdir", args="dirname=" + dir_path).read(), 'utf-8')))
 
-    def is_installed(self, sopid: str) -> bool:
+    def is_installed(self, sopid: str, syberdroid: bool = False) -> bool:
         """
-        判断指定应用是否已安装。\n
+        判断指定应用是否已经安装。\n
         :param sopid: 指定应用的sopid
+        :param syberdroid: 是否为安卓兼容应用，默认为否
         :return: 已安装返回True，否则为False
         """
-        return bool(int(str(self.device.con.get(path="isAppInstalled", args="sopid=" + sopid).read(), 'utf-8')))
+        _fi = self.device.get_framework_info()
+        if syberdroid and (_fi == {} or not _fi['syberdroid']):
+            return False
+        return bool(int(str(self.device.con.get(path="isAppInstalled", args="sopid=" + sopid + "&androidapp="
+                                                                            + str(int(syberdroid))).read(), 'utf-8')))
 
     def is_uninstallable(self, sopid: str) -> bool:
         """
@@ -440,25 +691,35 @@ class Events:
         """
         return bool(int(str(self.device.con.get(path="isAppUninstallable", args="sopid=" + sopid).read(), 'utf-8')))
 
-    def install(self, file_path: str) -> bool:
+    def install(self, file_path: str, syberdroid: bool = False) -> bool:
         """
         安装应用。\n
-        :param file_path: 控制端.sop文件的路径
+        :param file_path: 控制端.sop(.apk)文件的路径
+        :param syberdroid: 是否为安卓兼容应用，默认为否
         :return: 成功返回True，否则为False
         """
+        _fi = self.device.get_framework_info()
+        if syberdroid and (_fi == {} or not _fi['syberdroid']):
+            return False
         if self.upload_file(file_path, "/tmp/"):
             file_name = file_path.split("/")[len(file_path.split("/")) - 1]
-            self.device.con.get(path="install", args="filepath=/tmp/" + file_name)
+            self.device.con.get(path="install", args="filepath=/tmp/" + file_name + "&androidapp="
+                                                     + str(int(syberdroid)))
             return True
         return False
 
-    def uninstall(self, sopid: str) -> bool:
+    def uninstall(self, sopid: str, syberdroid: bool = False) -> bool:
         """
         卸载应用。\n
         :param sopid: 要卸载的应用sopid
+        :param syberdroid: 是否为安卓兼容应用，默认为否
         :return: 成功返回True，否则为False
         """
-        return bool(int(str(self.device.con.get(path="uninstall", args="sopid=" + sopid).read(), 'utf-8')))
+        _fi = self.device.get_framework_info()
+        if syberdroid and (_fi == {} or not _fi['syberdroid']):
+            return False
+        return bool(int(str(self.device.con.get(path="uninstall", args="sopid=" + sopid + "&androidapp="
+                                                                       + str(int(syberdroid))).read(), 'utf-8')))
 
     def system_time(self) -> int:
         """
@@ -497,13 +758,18 @@ class Events:
         """
         return str(self.device.con.get(path="getLatestToast").read(), 'utf-8')
 
-    def clear_app_data(self, sopid: str) -> bool:
+    def clear_app_data(self, sopid: str, syberdroid: bool = False) -> bool:
         """
         清除应用数据。\n
         :param sopid: 应用sopid
+        :param syberdroid: 是否为安卓兼容应用，默认为否
         :return: 成功返回True，否则为False
         """
-        return self.__reply_status_check(self.device.con.get(path="clearAppData", args="sopid=" + sopid))
+        _fi = self.device.get_framework_info()
+        if syberdroid and (_fi == {} or not _fi['syberdroid']):
+            return False
+        return self.__reply_status_check(self.device.con.get(path="clearAppData", args="sopid=" + sopid + "&androidapp="
+                                                                                       + str(int(syberdroid))))
 
     def get_panel_state(self) -> bool:
         """
@@ -530,16 +796,22 @@ class Events:
             return self.__reply_status_check(self.device.con.get(path="setPanelState"))
         return True
 
-    def launch(self, sopid: str, uiappid: str, timeout: int = None) -> bool:
+    def launch(self, sopid: str, uiappid: str, timeout: int = None, syberdroid: bool = False) -> bool:
         """
         启动应用。\n
         :param sopid: 应用sopid
         :param uiappid: 应用uiappid
         :param timeout: 超时时间(单位:秒)，默认为框架超时时间
+        :param syberdroid: 是否为安卓兼容应用，默认为否
         :return: 成功返回True，否则为False
         """
+        _fi = self.device.get_framework_info()
+        if syberdroid and (_fi == {} or not _fi['syberdroid']):
+            return False
         self.device.refresh_layout()
-        self.device.con.get(path="launchApp", args="sopid=" + sopid + "&" + "uiappid=" + uiappid)
+        self.device.con.get(path="launchApp", args="sopid=" + sopid + "&uiappid=" + uiappid + "&androidapp="
+                                                   + str(int(syberdroid)))
+        sleep(1)
         if timeout is None:
             timeout = self.device.default_timeout
         die_time = int(time.time()) + timeout
@@ -548,41 +820,65 @@ class Events:
                 try:
                     self.device.refresh_layout()
                     selector = etree.XML(self.device.xml_string.encode('utf-8'))
-                    if selector.get("sopId") == sopid:
-                        return True
+                    _fi = self.device.get_framework_info()
+                    if _fi != {} and _fi['version_build'] < 241219:
+                        if selector.get("sopId") == sopid:
+                            return True
+                    else:
+                        if selector.get("sopId") == sopid:
+                            if not syberdroid or selector.get("androidApp") == "1":
+                                return True
+                        else:
+                            if syberdroid and selector.get("androidApp") == "1" and selector.get("sopId") == "com.android.permissioncontroller":
+                                return True
+                            if not syberdroid and selector.get("androidApp") != "1" and selector.get("sopId") == "systemui(FAKE_VALUE)":
+                                return True
                     break
                 except etree.XMLSyntaxError:
                     continue
-            self.device.con.get(path="launchApp", args="sopid=" + sopid + "&" + "uiappid=" + uiappid)
             sleep(1)
         return False
 
-    def close(self, sopid: str, uiappid: str) -> bool:
+    def close(self, sopid: str, uiappid: str, syberdroid: bool = False) -> bool:
         """
         关闭应用。\n
         :param sopid: 应用sopid
         :param uiappid: 应用uiappid
+        :param syberdroid: 是否为安卓兼容应用，默认为否
         :return: 成功返回True，否则为False
         """
-        return self.__reply_status_check(
-            self.device.con.get(path="quitApp", args="sopid=" + sopid + "&uiappid=" + uiappid))
+        _fi = self.device.get_framework_info()
+        if syberdroid and (_fi == {} or not _fi['syberdroid']):
+            return False
+        for i in range(0, 50):
+            self.device.con.get(path="quitApp", args="sopid=" + sopid + "&uiappid=" + uiappid + "&androidapp="
+                                                     + str(int(syberdroid)))
+            sleep(0.01)
+            if not self.app_is_running(sopid, syberdroid):
+                return True
+        return not self.app_is_running(sopid, syberdroid)
 
-    def is_topmost(self, sopid: str) -> bool:
+    def app_is_running(self, sopid: str, syberdroid: bool = False) -> bool:
+        """
+        查询应用是否正在运行。\n
+        :param sopid: 应用sopid
+        :param syberdroid: 是否为安卓兼容应用，默认为否
+        :return: 应用正在运行返回True，否则为False
+        """
+        _fi = self.device.get_framework_info()
+        if syberdroid and (_fi == {} or not _fi['syberdroid']):
+            return False
+        return bool(int(str(self.device.con.get(path="appIsRunning", args="sopid=" + sopid + "&androidapp=" + str(int(syberdroid))).read(), 'utf-8')))
+
+    def is_topmost(self, sopid: str, syberdroid: bool = False) -> bool:
         """
         判断指定应用是否显示在屏幕最上层。\n
         :param sopid: 指定应用的sopid
+        :param syberdroid: 是否为安卓兼容应用，默认为否
         :return: 在最上层返回True，否则为False
         """
-        for i in range(0, 10):
-            try:
-                self.device.refresh_layout()
-                selector = etree.XML(self.device.xml_string.encode('utf-8'))
-                if selector.get("sopId") == sopid:
-                    return True
-                break
-            except etree.XMLSyntaxError:
-                continue
-        return False
+        info = self.device.get_topmost_info()
+        return info['sopid'] == sopid and info['syberdroid'] == syberdroid
 
     def get_volume(self, role_type: AudioManagerRoleType) -> int:
         """
@@ -695,7 +991,7 @@ class Events:
         :return: 成功返回True，否则为False
         """
         return self.__reply_status_check(
-            self.device.con.get(path="setAutoBrightness", args="enable=" + str(enable)))
+            self.device.con.get(path="setAutoBrightness", args="enable=" + str(int(enable))))
 
     def get_auto_brightness(self) -> bool:
         """
@@ -772,6 +1068,41 @@ class Events:
                                                                          + "&timeout="
                                                                          + str(timeout)).read()))
 
+    def connect_psk_wifi(self, ssid: str, password: str, timeout: int = None) -> bool:
+        """
+        连接设备至PSK加密的WiFi网络。\n
+        :param ssid: WiFi网络名称
+        :param password: WIFI密码
+        :param timeout: 超时时间(单位:秒)，默认为框架超时时间
+        :return: 成功返回True，否则为False
+        """
+        _fi = self.device.get_framework_info()
+        if _fi != {} and _fi['version_build'] < 241225:
+            return False
+        if len(password) < 8:
+            return False
+        to_str = "&timeout=" + str(timeout) if timeout else ""
+        return bool(int(self.device.con.get(path="connectPskWiFi", args="ssid=" + ssid + "&password=" + password
+                                                                         + to_str).read()))
+
+    def set_wlan_enabled(self, enable: bool) -> bool:
+        """
+        设置设备WLAN功能是否开启。
+        """
+        _fi = self.device.get_framework_info()
+        if _fi != {} and _fi['version_build'] < 241225:
+            return False
+        return bool(int(self.device.con.get(path="setWlanEnabled", args="enabled=" + str(int(enable))).read()))
+
+    def get_wlan_enabled(self) -> bool:
+        """
+        获取设备WLAN功能是否开启。
+        """
+        _fi = self.device.get_framework_info()
+        if _fi != {} and _fi['version_build'] < 241225:
+            return False
+        return bool(int(self.device.con.get(path="getWlanEnabled").read()))
+
     def password_exists(self) -> bool:
         """
         查询设备是否存在锁屏密码。\n
@@ -806,3 +1137,113 @@ class Events:
                           is_charging=bool(json_obj["is_charging"]),
                           battery_level=int(json_obj["battery_level"]))
         return stat
+
+    def set_app_permission(self, sopid: str, perm: str, enabled: bool, syberdroid: bool = False) -> bool:
+        """
+        设置应用权限。\n
+        :param sopid: 应用包名
+        :param perm: 权限名称
+        :param enabled: 是否开启权限
+        :param syberdroid: 是否为安卓兼容应用，默认为否
+        :return: 成功返回True，否则为False
+        """
+        _fi = self.device.get_framework_info()
+        if _fi != {} and _fi['version_build'] < 241219:
+            return False
+        if syberdroid and (_fi == {} or not _fi['syberdroid'] or _fi['version_build'] < 250118):
+            return False
+        if perm not in self.device.permissions().keys():
+            print("set_app_permission: permission not found!")
+            return False
+        return self.__reply_status_check(
+            self.device.con.get(path="setAppPermission",
+                                args="sopid=" + sopid + "&perm=" + perm + "&enabled=" + str(int(enabled))
+                                     + "&androidapp=" + str(int(syberdroid))))
+
+    def set_app_permissions(self, sopid: str, perm_list: list, enabled: bool, syberdroid: bool = False) -> bool:
+        """
+        设置应用权限。\n
+        :param sopid: 应用包名
+        :param perm_list: 权限名称列表
+        :param enabled: 是否开启权限
+        :param syberdroid: 是否为安卓兼容应用，默认为否
+        :return: 成功返回True，否则为False
+        """
+        _fi = self.device.get_framework_info()
+        if _fi != {} and _fi['version_build'] < 241219:
+            return False
+        if syberdroid and (_fi == {} or not _fi['syberdroid'] or _fi['version_build'] < 250118):
+            return False
+        if len(perm_list) == 0:
+            print("set_app_permissions: permission list is empty!")
+            return False
+        for perm in perm_list:
+            if perm not in self.device.permissions().keys():
+                print("set_app_permissions: permission \"" + perm + "\" not found!")
+                return False
+        return self.__reply_status_check(
+            self.device.con.get(path="setAppPermissions",
+                                args="sopid=" + sopid + "&perms=" + "|".join(perm_list) + "&enabled=" + str(int(enabled))
+                                     + "&androidapp=" + str(int(syberdroid))))
+
+    def get_app_permissions(self, sopid: str, syberdroid: bool = False) -> dict:
+        """
+        获取应用权限。\n
+        :param sopid: 应用包名
+        :param syberdroid: 是否为安卓兼容应用，默认为否
+        :return: 权限字典，key为权限名称，value为字典类型(包含中英文说明以及授权状态，当查询安卓应用时则是中文名称、说明、授权状态以及安卓标识)
+        """
+        _fi = self.device.get_framework_info()
+        if _fi != {} and _fi['version_build'] < 241219:
+            return {}
+        if syberdroid and (_fi == {} or not _fi['syberdroid'] or _fi['version_build'] < 250118):
+            return {}
+        json_str = str(self.device.con.get(path="getAppPermissions",
+                                           args="sopid=" + sopid + "&androidapp=" + str(int(syberdroid))).read(), 'utf-8')
+        return json.loads(json_str)
+
+    def check_app_permission(self, sopid: str, perm: str, syberdroid: bool = False) -> bool:
+        """
+        检查应用权限。\n
+        :param sopid: 应用包名
+        :param perm: 权限名称
+        :param syberdroid: 是否为安卓兼容应用，默认为否
+        :return: 是否开启权限
+        """
+        _fi = self.device.get_framework_info()
+        if _fi != {} and _fi['version_build'] < 241219:
+            return False
+        if syberdroid and (_fi == {} or not _fi['syberdroid'] or _fi['version_build'] < 250118):
+            return False
+        return bool(int(self.device.con.get(path="checkAppPermission", args="sopid=" + sopid + "&perm=" + perm
+                                                                            + "&androidapp=" + str(int(syberdroid))).read()))
+
+    def enable_all_permissions(self, sopid: str, syberdroid: bool = False) -> bool:
+        """
+        启用应用所有权限。\n
+        :param sopid: 应用包名
+        :param syberdroid: 是否为安卓兼容应用，默认为否
+        :return: 成功返回True，否则为False
+        """
+        _fi = self.device.get_framework_info()
+        if _fi != {} and _fi['version_build'] < 241219:
+            return False
+        if syberdroid and (_fi == {} or not _fi['syberdroid'] or _fi['version_build'] < 250118):
+            return False
+        return self.__reply_status_check(self.device.con.get(path="enableAllPermissions",
+                                                             args="sopid=" + sopid + "&androidapp=" + str(int(syberdroid))))
+
+    def disable_all_permissions(self, sopid: str, syberdroid: bool = False) -> bool:
+        """
+        禁用应用所有权限。\n
+        :param sopid: 应用包名
+        :param syberdroid: 是否为安卓兼容应用，默认为否
+        :return: 成功返回True，否则为False
+        """
+        _fi = self.device.get_framework_info()
+        if _fi != {} and _fi['version_build'] < 241219:
+            return False
+        if syberdroid and (_fi == {} or not _fi['syberdroid'] or _fi['version_build'] < 250118):
+            return False
+        return self.__reply_status_check(self.device.con.get(path="disableAllPermissions",
+                                                             args="sopid=" + sopid + "&androidapp=" + str(int(syberdroid))))
