@@ -14,6 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 import base64
+import urllib3.exceptions
 from PIL import Image
 from io import BytesIO
 import time
@@ -38,7 +39,7 @@ html_string_1 = "<!DOCTYPE html><html lang=\"en\"><head><meta charset=\"UTF-8\">
                 "</title></head><body style=\"background:#000;clip:rect(auto,auto,auto,auto);" \
                 "\"><div id=\"content\" style=\"width:"
 html_string_2 = "px;height:"
-html_string_3 = "px;clip:rect(auto,auto,auto,auto);\">"
+html_string_3 = "px;clip:rect(auto,auto,auto,auto);position:absolute;overflow:hidden;\">"
 html_string_4 = "</div></body></html>"
 
 
@@ -107,6 +108,7 @@ class Item:
     def __refresh_node(self):
         self.device.refresh_layout()
         if self.xpath != "":
+            self.node = None
             for i in range(0, 10):
                 try:
                     selector = etree.XML(self.device.xml_string.encode('utf-8'))
@@ -258,7 +260,7 @@ class Item:
         if self.node.getparent() is None:
             return None
         pn = self.node.getprevious()
-        while previous_i:
+        while pn is not None:
             if pn.tag == self.node.tag:
                 return Item(self.device, self.sopid, "", pn)
             pn = pn.getprevious()
@@ -291,7 +293,7 @@ class Item:
         if self.node.getparent() is None:
             return None
         nn = self.node.getnext()
-        while nn:
+        while nn is not None:
             if nn.tag == self.node.tag:
                 return Item(self.device, self.sopid, "", nn)
             nn = nn.getnext()
@@ -571,13 +573,23 @@ class Item:
                                                    c_char_p(bytes(self.__attributes["temp_id"], "utf-8")),
                                                    c_int(self.__display_width),
                                                    c_int(self.__display_height))
-                self.device.webdriver.get("data:text/html;charset=utf-8," +
-                                          html_string_1 + str(self.__display_width) +
-                                          html_string_2 + str(self.__display_height) +
-                                          html_string_3 + string_at(html_string, -1).decode('utf-8') +
-                                          html_string_4)
-                image = cv2.imdecode(np.frombuffer(base64.b64decode(self.device.webdriver.get_screenshot_as_base64()),
-                                                   np.uint8), cv2.COLOR_RGB2BGR)
+                if self.device.is_main:
+                    self.device.conn_phantomjs_before()
+                try:
+                    self.device.webdriver.get("data:text/html;charset=utf-8," +
+                                              html_string_1 + str(self.__display_width) +
+                                              html_string_2 + str(self.__display_height) +
+                                              html_string_3 + string_at(html_string, -1).decode('utf-8') +
+                                              html_string_4)
+                    image = cv2.imdecode(np.frombuffer(base64.b64decode(self.device.webdriver.get_screenshot_as_base64()),
+                                                       np.uint8), cv2.COLOR_RGB2BGR)
+                except (cv2.error, urllib3.exceptions.ProtocolError) as e:
+                    print(e)
+                    if self.device.is_main:
+                        self.device.conn_phantomjs_after()
+                    return []
+                if self.device.is_main:
+                    self.device.conn_phantomjs_after()
             # win_name = str(self.device.system_time())
             # cv2.namedWindow(win_name, cv2.WINDOW_NORMAL)
             # cv2.imshow(win_name, image)
@@ -620,7 +632,7 @@ class Item:
         scale = abs(float(node.getAttribute("scale")) * fs)
         height = round(float(node.getAttribute("height")) * scale)
         width = round(float(node.getAttribute("width")) * scale)
-        rotation = 0 - int(node.getAttribute("rotation"))
+        rotation = 0 - int(round(float(node.getAttribute("rotation"))))
         has_contents = bool(int(node.getAttribute("hasContents")))
         cx = round(float(node.getAttribute("centerXToGlobal")))
         cy = round(float(node.getAttribute("centerYToGlobal")))
@@ -951,11 +963,20 @@ class Item:
         判断元素控件是否显示，显示则执行点击操作。\n
         :param delay: 点击延时时间(单位:毫秒), 默认无延时
         :param timeout: 超时时间(单位:秒)，默认为框架超时时间
-        :return: 点击成功返回True，否则返回False，若不存在则中断
+        :return: 点击成功返回True，失败或不存在返回False
         """
         if self.exist(timeout):
             return self.device.click(Point(self.rect[0][1]+1, self.rect[0][0]+1), delay)
-        assert False
+        return False
+
+    def drag(self, p: Point, delay: int = 1):
+        """
+        将元素控件拖动到指定点位。\n
+        :param p: 坐标点类Point对象，目标点
+        :param delay: 起始点长按时间，默认1秒
+        :return: 成功返回True，否则为False
+        """
+        return self.device.drag(Point(self.center_x_to_global(), self.center_y_to_global()), p, delay)
 
     def submit_string(self, text: str) -> bool:
         """
@@ -964,6 +985,7 @@ class Item:
         :return: 成功返回True，否则为False
         """
         self.click()
+        sleep(1)
         return self.device.submit_string(text)
 
     def grab_image_to_base64(self) -> str:
@@ -976,10 +998,11 @@ class Item:
                                                 self.__attributes["width"], self.__attributes["height"],
                                                 self.__attributes["rotation"], self.__attributes["scale"])
 
-    def contrast_picture(self, path: str) -> float:
+    def contrast_picture(self, path: str, scale: bool = False) -> float:
         """
         使用本地的图片文件与当前元素控件截图进行图片比对。\n
         :param path: 本地的图片路径
+        :param scale: 对比之前是否通过缩放来统一二者尺寸，默认为否
         :return: 对比值，值越小越相似
         """
         self.__refresh_node()
@@ -988,24 +1011,30 @@ class Item:
             return 999999.9
         current_pic = Image.open(BytesIO(base64.b64decode(current_pic_base64)))
         target_pic = Image.open(path)
+        if scale:
+            cw, ch = current_pic.size
+            tw, th = target_pic.size
+            if cw > tw:
+                nw = tw
+            else:
+                nw = cw
+            if ch > th:
+                nh = th
+            else:
+                nh = ch
+            current_pic = current_pic.resize((nw, nh))
+            target_pic = target_pic.resize((nw, nh))
         h1 = current_pic.histogram()
         h2 = target_pic.histogram()
         result = math.sqrt(reduce(operator.add, list(map(lambda a, b: (a - b) ** 2, h1, h2))) / len(h1))
         return result
 
-    def contrast_picture_from_base64(self, pic: str) -> float:
+    def contrast_picture_from_base64(self, pic: str, scale: bool = False) -> float:
         """
         使用图片的base64字符串与当前元素控件截图进行图片比对。\n
         :param pic: 图片的base64字符串
+        :param scale: 对比之前是否通过缩放来统一二者尺寸，默认为否
         :return: 对比值，值越小越相似
         """
         self.__refresh_node()
-        current_pic_base64 = self.grab_image_to_base64()
-        if current_pic_base64 == "" or pic == "":
-            return 999999.9
-        current_pic = Image.open(BytesIO(base64.b64decode(current_pic_base64)))
-        target_pic = Image.open(BytesIO(base64.b64decode(pic)))
-        h1 = current_pic.histogram()
-        h2 = target_pic.histogram()
-        result = math.sqrt(reduce(operator.add, list(map(lambda a, b: (a - b) ** 2, h1, h2))) / len(h1))
-        return result
+        return self.device.contrast_picture_from_base64(pic, self.grab_image_to_base64(), scale)
