@@ -18,6 +18,7 @@ import psutil
 from typing import TypeVar
 from .DataStruct import WatcherActive
 from .DataStruct import Keys
+from .selenium_phantomjs.webdriver.remote import webdriver
 
 
 WatchContext_T = TypeVar('WatchContext_T', bound='WatchContext')
@@ -30,23 +31,51 @@ class WatchWorker:
         self.conn = conn
         self.main_pid = main_pid
         self.__watcher_list = []
+        self.__watcher_list_raw = {}
 
     def __get_list(self):
         if self.conn.poll():
             data = dict(self.conn.recv())
-            if 'watcher_list' in data.keys():
-                self.__watcher_list = data['watcher_list']
+            if data['action'] == 'create':
+                if data['object'] not in self.__watcher_list_raw.keys():
+                    self.__watcher_list_raw[data['object']] = {}
+                self.__watcher_list_raw[data['object']][data['watcher_name']] = data['watcher_data']
+            elif data['action'] == 'start':
+                if data['object'] in self.__watcher_list_raw.keys():
+                    if data['watcher_name'] in self.__watcher_list_raw[data['object']].keys():
+                        if 'is_run' in self.__watcher_list_raw[data['object']][data['watcher_name']].keys():
+                            self.__watcher_list_raw[data['object']][data['watcher_name']]['is_run'] = True
+            elif data['action'] == 'pause':
+                if data['object'] in self.__watcher_list_raw.keys():
+                    if data['watcher_name'] in self.__watcher_list_raw[data['object']].keys():
+                        if 'is_run' in self.__watcher_list_raw[data['object']][data['watcher_name']].keys():
+                            self.__watcher_list_raw[data['object']][data['watcher_name']]['is_run'] = False
+            elif data['action'] == 'delete':
+                if data['object'] in self.__watcher_list_raw.keys():
+                    if data['watcher_name'] in self.__watcher_list_raw[data['object']].keys():
+                        del self.__watcher_list_raw[data['object']][data['watcher_name']]
+            elif data['action'] == 'clear':
+                if data['object'] in self.__watcher_list_raw.keys():
+                    del self.__watcher_list_raw[data['object']]
+            elif data['action'] == 'update_wd_port':
+                if data['port'] != self.device.wd_port:
+                    self.device.wd_port = data['port']
+                    self.device.webdriver = webdriver.WebDriver(command_executor='http://127.0.0.1:'
+                                                                                 + str(self.device.wd_port) + '/wd/hub')
+            self.__watcher_list = []
+            for obj in self.__watcher_list_raw.keys():
+                for watcher in self.__watcher_list_raw[obj].values():
+                    self.__watcher_list.append(watcher)
 
     def run(self):
         main_process = psutil.Process(self.main_pid)
         while True:
             time.sleep(1)
-            self.__get_list()
-            if len(self.__watcher_list) == 0:
-                continue
-            main_process.suspend()
             try:
+                self.__get_list()
                 self.device.refresh_layout()
+                if len(self.__watcher_list) == 0:
+                    continue
                 for watcher in self.__watcher_list:
                     if not watcher['is_run']:
                         continue
@@ -55,6 +84,7 @@ class WatchWorker:
                         if self.device.find_item_by_xpath(xpath['sop_id'], xpath['xpath']).exist(0):
                             flag += 1
                     if flag == len(watcher['xpath_list']):
+                        main_process.suspend()
                         for xpath in watcher['xpath_list']:
                             if xpath['index'] == watcher['index']:
                                 if watcher['active'] == WatcherActive.CLICK:
@@ -67,11 +97,10 @@ class WatchWorker:
                                     self.device.launch(watcher['active_sop_id'], watcher['active_ui_app_id'])
                                 elif watcher['active'] == WatcherActive.STOP:
                                     self.device.close(watcher['active_sop_id'], watcher['active_ui_app_id'])
+                        main_process.resume()
             except Exception as e:
-                print(e)
                 main_process.resume()
                 continue
-            main_process.resume()
 
 
 class Watcher:
@@ -79,8 +108,9 @@ class Watcher:
     监视者类。
     """
 
-    def __init__(self, data: dict, d):
-        self.__watcher_data = data
+    def __init__(self, name: str, is_run: bool, d):
+        self.__watcher_data = {'is_run': is_run}
+        self.__watcher_name = name
         self.device = d
 
     def when(self, sop_id: str, xpath_key: str) -> WatchContext_T:
@@ -97,7 +127,7 @@ class Watcher:
                                                   'xpath': self.device.get_xpath(sop_id, xpath_key),
                                                   'sop_id': sop_id})
         self.__watcher_data['index'] = index
-        return WatchContext(self.__watcher_data, self.device)
+        return WatchContext(self.__watcher_name, self.__watcher_data, self.device)
 
 
 class WatchContext:
@@ -105,8 +135,9 @@ class WatchContext:
     监视者上下文类。
     """
 
-    def __init__(self, data: dict, d):
+    def __init__(self, name: str, data: dict, d):
         self.__watcher_data = data
+        self.__watcher_name = name
         self.device = d
 
     def when(self, sop_id: str, xpath_key: str) -> WatchContext_T:
@@ -168,6 +199,4 @@ class WatchContext:
         self.__push()
 
     def __push(self):
-        self.__watcher_data['is_run'] = False
-        self.device.watcher_list.append(self.__watcher_data)
-        self.device.push_watcher_data()
+        self.device.push_watcher(self.__watcher_name, self.__watcher_data)
